﻿using System;
using System.Linq;
using System.Windows;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WpfDesktopEdition;

namespace NoaaWeatherSample
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        static MainWindow()
        {
            NoaaRadarMonitor.RefreshInterval = new TimeSpan(0, 0, 30);

            // Monitor weather radar service.
            NoaaRadarMonitor.StartMonitoring();

            NoaaWeatherStationMonitor.RefreshInterval = new TimeSpan(0, 0, 30);

            // Monitor weather station service.
            NoaaWeatherStationMonitor.StartMonitoring();
        }

        private void WpfMap_Loaded(object sender, RoutedEventArgs e)
        {
            Map1.MapUnit = GeographyUnit.DecimalDegree;
            Map1.CurrentExtent = new RectangleShape(-155.733, 95.60, 104.42, -81.9);
            WorldMapKitWmsWpfOverlay worldOverlay = new WorldMapKitWmsWpfOverlay();
            Map1.Overlays.Add("WMK", worldOverlay);
            Map1.Refresh();
        }

        private void RadarButton_Click(object sender, RoutedEventArgs e)
        {
            AddNoaaWeatherLayer<NoaaRadarRasterLayer>(radarLayer =>
            {
                radarLayer.Transparency = 50 * 2.55f;
            });
        }

        private void StationButton_Click(object sender, RoutedEventArgs e)
        {
            AddNoaaWeatherLayer<NoaaWeatherStationFeatureLayer>(stationLayer =>
            {
                NoaaWeatherStationStyle style = new NoaaWeatherStationStyle();
                stationLayer.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(style);
                stationLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            });
        }

        private void AddNoaaWeatherLayer<T>(Action<T> setLayerPropertiesAction = null) where T : Layer, new()
        {
            LayerOverlay layerOverlay = GetLayerOverlay();
            if (!layerOverlay.Layers.Any(l => l is T))
            {
                T layer = new T();
                if (setLayerPropertiesAction != null)
                {
                    setLayerPropertiesAction(layer);
                }
                layerOverlay.Layers.Add(layer);
                layerOverlay.Refresh();
            }
        }

        private LayerOverlay GetLayerOverlay()
        {
            LayerOverlay layerOverlay = Map1.Overlays.OfType<LayerOverlay>().FirstOrDefault();
            if (layerOverlay == null)
            {
                layerOverlay = new LayerOverlay();
                Map1.Overlays.Add(layerOverlay);
                Map1.Refresh(layerOverlay);
            }

            return layerOverlay;
        }
    }
}